# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-03 11:26
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Index',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField()),
                ('unique', models.BooleanField()),
                ('field', django.contrib.postgres.fields.jsonb.JSONField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
    ]



import django.contrib.postgres.fields as pg
from django.contrib.contenttypes.models import ContentType
from django.db import models


class Index(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    name = models.TextField()
    unique = models.BooleanField()
    field = pg.JSONField()

    def get_structure(self):
        return {"name": self.name, "fields": set(self.field), "unique": self.unique}



from typing import List


class JSONIndex:
    def __init__(self, field: List[str], name: str, unique=False, concurrently=True):
        self.field = field
        self.name = name
        self.concurrently = concurrently
        self.unique = unique

    def get_structure(self):
        if type(self.field[0]) is list or type(self.field[0]) is tuple:
            fields = set(self.get_field(f) for f in self.field)
        else:
            fields = {self.get_field(self.field)}
        return {"name": self.name, "fields": fields, "unique": self.unique}

    def get_sql(self, db_table: str):
        concurrently = "CONCURRENTLY" if self.concurrently else ""
        structure = self.get_structure()
        unique = "UNIQUE" if structure["unique"] else ""
        fields = structure["fields"]
        if type(fields) is set:
            fields = ", ".join(fields)
            fields = f"({fields})"

        return f"""
                CREATE {unique} INDEX {concurrently}
                IF NOT EXISTS {self.name}
                ON {db_table} {fields}
        """

    @staticmethod
    def get_field(field: List[str]):
        field = field[:]
        for i, part in enumerate(field[1:], 1):
            field[i] = f"'{part}'"
        field = "->".join(field[:-1]) + "->>" + field[-1]
        return f"({field})"

